/*
** HDL4SE: VerilogۺϷƽ̨
** Copyright (C) 2021-2021, raoxianhong<raoxianhong@163.net>
** LCOM: ģ
** Copyright (C) 2021-2021, raoxianhong<raoxianhong@163.net>
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**
** * Redistributions of source code must retain the above copyright notice,
**   this list of conditions and the following disclaimer.
** * Redistributions in binary form must reproduce the above copyright notice,
**   this list of conditions and the following disclaimer in the documentation
**   and/or other materials provided with the distribution.
** * The name of the author may be used to endorse or promote products
**   derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
** LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
** CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
** SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
** INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
** CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
** ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
** THE POSSIBILITY OF SUCH DAMAGE.
*/

/*
* verilog_keyword.c
  ޸ļ¼
	202105280619: rxh, initial version
*/

#include "verilog_keyword.h"
#include "verilog_parser.h"
#include "string.h"

#define DEFKEYWORD(id, n) {KW_##id, n}

static struct s_key_word_list {
	int keyid;
	const char *name;
} key_word_list[] = {
  DEFKEYWORD(ALWAYS              ,"always"),
  DEFKEYWORD(AND                 ,"and"),
  DEFKEYWORD(ASSIGN              ,"assign"),
  DEFKEYWORD(AUTOMATIC           ,"automatic"),
  DEFKEYWORD(BEGIN               ,"begin"),
  DEFKEYWORD(BUF                 ,"buf"),
  DEFKEYWORD(BUFIF0              ,"bufif0"),
  DEFKEYWORD(BUFIF1              ,"bufif1"),
  DEFKEYWORD(CASE                ,"case"),
  DEFKEYWORD(CASEX               ,"casex"),
  DEFKEYWORD(CASEZ               ,"casez"),
  DEFKEYWORD(CELL                ,"cell"),
  DEFKEYWORD(CMOS                ,"cmos"),
  DEFKEYWORD(CONFIG              ,"config"),
  DEFKEYWORD(DEASSIGN            ,"deassign"),
  DEFKEYWORD(DEFAULT             ,"default"),
  DEFKEYWORD(DEFPARAM            ,"defparam"),
  DEFKEYWORD(DESIGN              ,"design"),
  DEFKEYWORD(DISABLE             ,"disable"),
  DEFKEYWORD(EDGE                ,"edge"),
  DEFKEYWORD(ELSE                ,"else"),
  DEFKEYWORD(END                 ,"end"),
  DEFKEYWORD(ENDCASE             ,"endcase"),
  DEFKEYWORD(ENDCONFIG           ,"endconfig"),
  DEFKEYWORD(ENDFUNCTION         ,"endfunction"),
  DEFKEYWORD(ENDGENERATE         ,"endgenerate"),
  DEFKEYWORD(ENDMODULE           ,"endmodule"),
  DEFKEYWORD(ENDPRIMITIVE        ,"endprimitive"),
  DEFKEYWORD(ENDSPECIFY          ,"endspecify"),
  DEFKEYWORD(ENDTABLE            ,"endtable"),
  DEFKEYWORD(ENDTASK             ,"endtask"),
  DEFKEYWORD(EVENT               ,"event"),
  DEFKEYWORD(FOR                 ,"for"),
  DEFKEYWORD(FORCE               ,"force"),
  DEFKEYWORD(FOREVER             ,"forever"),
  DEFKEYWORD(FORK                ,"fork"),
  DEFKEYWORD(FUNCTION            ,"function"),
  DEFKEYWORD(GENERATE            ,"generate"),
  DEFKEYWORD(GENVAR              ,"genvar"),
  DEFKEYWORD(HIGHZ0              ,"highz0"),
  DEFKEYWORD(HIGHZ1              ,"highz1"),
  DEFKEYWORD(IF                  ,"if"),
  DEFKEYWORD(IFNONE              ,"ifnone"),
  DEFKEYWORD(INCDIR              ,"-incdir"),
  DEFKEYWORD(INCLUDE             ,"include"),
  DEFKEYWORD(INITIAL             ,"initial"),
  DEFKEYWORD(INOUT               ,"inout"),
  DEFKEYWORD(INPUT               ,"input"),
  DEFKEYWORD(INSTANCE            ,"instance"),
  DEFKEYWORD(INTEGER             ,"integer"),
  DEFKEYWORD(JOIN                ,"join"),
  DEFKEYWORD(LARGE               ,"large"),
  DEFKEYWORD(LIBLIST             ,"liblist"),
  DEFKEYWORD(LIBRARY             ,"library"),
  DEFKEYWORD(LOCALPARAM          ,"localparam"),
  DEFKEYWORD(MACROMODULE         ,"macromodule"),
  DEFKEYWORD(MEDIUM              ,"medium"),
  DEFKEYWORD(MODULE              ,"module"),
  DEFKEYWORD(NAND                ,"nand"),
  DEFKEYWORD(NEGEDGE             ,"negedge"),
  DEFKEYWORD(NMOS                ,"nmos"),
  DEFKEYWORD(NOR                 ,"nor"),
  DEFKEYWORD(NOSHOWCANCELLED     ,"noshowcancelled"),
  DEFKEYWORD(NOT                 ,"not"),
  DEFKEYWORD(NOTIF0              ,"notif0"),
  DEFKEYWORD(NOTIF1              ,"notif1"),
  DEFKEYWORD(OR                  ,"or"),
  DEFKEYWORD(OUTPUT              ,"output"),
  DEFKEYWORD(PARAMETER           ,"parameter"),
  DEFKEYWORD(PMOS                ,"pmos"),
  DEFKEYWORD(POSEDGE             ,"posedge"),
  DEFKEYWORD(PRIMITIVE           ,"primitive"),
  DEFKEYWORD(PULL0               ,"pull0"),
  DEFKEYWORD(PULL1               ,"pull1"),
  DEFKEYWORD(PULLDOWN            ,"pulldown"),
  DEFKEYWORD(PULLUP              ,"pullup"),
  DEFKEYWORD(PULSESTYLE_ONEVENT  ,"pulsestyle_onevent"),
  DEFKEYWORD(PULSESTYLE_ONDETECT ,"pulsestyle_ondetect"),
  DEFKEYWORD(RCMOS               ,"rcmos"),
  DEFKEYWORD(REAL                ,"real"),
  DEFKEYWORD(REALTIME            ,"realtime"),
  DEFKEYWORD(REG                 ,"reg"),
  DEFKEYWORD(RELEASE             ,"release"),
  DEFKEYWORD(REPEAT              ,"repeat"),
  DEFKEYWORD(RNMOS               ,"rnmos"),
  DEFKEYWORD(RPMOS               ,"rpmos"),
  DEFKEYWORD(RTRAN               ,"rtran"),
  DEFKEYWORD(RTRANIF0            ,"rtranif0"),
  DEFKEYWORD(RTRANIF1            ,"rtranif1"),
  DEFKEYWORD(SCALARED            ,"scalared"),
  DEFKEYWORD(SHOWCANCELLED       ,"showcancelled"),
  DEFKEYWORD(SIGNED              ,"signed"),
  DEFKEYWORD(SMALL               ,"small"),
  DEFKEYWORD(SPECIFY             ,"specify"),
  DEFKEYWORD(SPECPARAM           ,"specparam"),
  DEFKEYWORD(STRONG0             ,"strong0"),
  DEFKEYWORD(STRONG1             ,"strong1"),
  DEFKEYWORD(SUPPLY0             ,"supply0"),
  DEFKEYWORD(SUPPLY1             ,"supply1"),
  DEFKEYWORD(TABLE               ,"table"),
  DEFKEYWORD(TASK                ,"task"),
  DEFKEYWORD(TIME                ,"time"),
  DEFKEYWORD(TRAN                ,"tran"),
  DEFKEYWORD(TRANIF0             ,"tranif0"),
  DEFKEYWORD(TRANIF1             ,"tranif1"),
  DEFKEYWORD(TRI0                ,"tri0"),
  DEFKEYWORD(TRI1                ,"tri1"),
  DEFKEYWORD(TRI                 ,"tri"),
  DEFKEYWORD(TRIAND              ,"triand"),
  DEFKEYWORD(TRIOR               ,"trior"),
  DEFKEYWORD(TRIREG              ,"trireg"),
  DEFKEYWORD(UNSIGNED            ,"unsigned"),
  DEFKEYWORD(USE                 ,"use"),
  DEFKEYWORD(VECTORED            ,"vectored"),
  DEFKEYWORD(WAIT                ,"wait"),
  DEFKEYWORD(WAND                ,"wand"),
  DEFKEYWORD(WEAK0               ,"weak0"),
  DEFKEYWORD(WEAK1               ,"weak1"),
  DEFKEYWORD(WHILE               ,"while"),
  DEFKEYWORD(WIRE                ,"wire"),
  DEFKEYWORD(WOR                 ,"wor"),
  DEFKEYWORD(XNOR                ,"xnor"),
  DEFKEYWORD(XOR                 ,"xor"),
};

int verilog_find_reserved_word(const char * ident)
{
	int i;
	for (i = 0;i<sizeof(key_word_list)/sizeof(key_word_list[0]);i++) {
		if (strcmp(ident, key_word_list[i].name) == 0)
			return key_word_list[i].keyid;
	}
	return -1;
}

