%require  "3.0"
%defines 
%define parse.trace
%verbose

%define parse.error verbose
 
%{
    #include <stdio.h> 
    #include <string.h>
    #include <assert.h>
    #include "verilog_keyword.h"

    extern int yylex();
    extern int yylineno;
    extern char * yytext;

    void yyerror(const char *msg){
        printf("line %d - ERROR: %s\n", yylineno,msg);
        printf("- '%s'\n", yytext);
    }
%} 
 
%code requires{
#include "stdio.h"
#include "object.h"
#include "dlist.h"
#include "verilog_parsetree.h"
#include "verilog_root.h"
#include "verilog_module.h"
#include "verilog_keyword.h"

}

/* token types */
%union {
	HOBJECT treenode;
	char * string;
	int token;
	int operator;
	int ival;
	IDListVar list;
}

%token <string> END  ANY  NEWLINE  SPACE  TAB

%token <string> BIN_VALUE  OCT_VALUE  HEX_VALUE  DEC_VALUE  
				DEC_BASE  BIN_BASE  OCT_BASE  HEX_BASE  
				NUM_REAL  NUM_SIZE  UNSIGNED_NUMBER  
				SYSTEM_ID  SIMPLE_ID  ESCAPED_ID  DEFINE_ID  
				STRING

/* Operators Precedence */
%token <token> '@' ',' '#' '.' '=' ':' ';' '(' ')' '[' ']' '{' '}' '+' '-' '*' '/' '%' '>' '<' '!' '&' '|' '^' '~' '?' 

%token <token> ATTRIBUTE_START ATTRIBUTE_END ASL ASR LSL LSR GTE LTE L_AND L_OR L_EQ C_EQ L_NEQ C_NEQ B_EQU B_NAND B_NOR IDX_PRT_SEL

%token <token>  KW_ALWAYS  KW_AND  KW_ASSIGN  KW_AUTOMATIC  KW_BEGIN  KW_BUF  KW_BUFIF0  KW_BUFIF1  KW_CASE  KW_CASEX  
				KW_CASEZ  KW_CELL  KW_CMOS  KW_CONFIG  KW_DEASSIGN  KW_DEFAULT  KW_DEFPARAM  KW_DESIGN  KW_DISABLE  
				KW_EDGE  KW_ELSE  KW_END  KW_ENDCASE  KW_ENDCONFIG  KW_ENDFUNCTION  KW_ENDGENERATE  KW_ENDMODULE  
				KW_ENDPRIMITIVE  KW_ENDSPECIFY  KW_ENDTABLE  KW_ENDTASK  KW_EVENT  KW_FOR  KW_FORCE  KW_FOREVER  KW_FORK  
				KW_FUNCTION  KW_GENERATE  KW_GENVAR  KW_HIGHZ0  KW_HIGHZ1  KW_IF  KW_IFNONE  KW_INCDIR  KW_INCLUDE  
				KW_INITIAL  KW_INOUT  KW_INPUT  KW_INSTANCE  KW_INTEGER  KW_JOIN  KW_LARGE  KW_LIBLIST  KW_LIBRARY  
				KW_LOCALPARAM  KW_MACROMODULE  KW_MEDIUM  KW_MODULE  KW_NAN  KW_NEGEDGE  KW_NMOS  KW_NOR  KW_NOSHOWCANCELLED  
				KW_NOT  KW_NOTIF0  KW_NOTIF1  KW_OR  KW_OUTPUT  KW_PARAMETER  KW_PATHPULSE  KW_PMOS  KW_POSEDGE  KW_PRIMITIVE  
				KW_PULL0  KW_PULL1  KW_PULLDOWN  KW_PULLUP  KW_PULSESTYLE_ONEVENT  KW_PULSESTYLE_ONDETECT  KW_RCMOS  
				KW_REAL  KW_REALTIME  KW_REG  KW_RELEASE  KW_REPEAT  KW_RNMOS  KW_RPMOS  KW_RTRAN  KW_RTRANIF0  KW_RTRANIF1  
				KW_SCALARED  KW_SHOWCANCELLED  KW_SIGNED  KW_SMALL  KW_SPECIFY  KW_SPECPARAM  KW_STRONG0  KW_STRONG1  
				KW_SUPPLY0  KW_SUPPLY1  KW_TABLE  KW_TASK  KW_TIME  KW_TRAN  KW_TRANIF0  KW_TRANIF1  KW_TRI  KW_TRI0  
				KW_TRI1  KW_TRIAND  KW_TRIOR  KW_TRIREG  KW_UNSIGNED  KW_USE  KW_VECTORED  KW_WAIT  KW_WAND  KW_WEAK0  
				KW_WEAK1  KW_WHILE  KW_WIRE  KW_WOR  KW_XNOR  KW_XOR KW_NAND

/* Operator Precedence */


%right  '?'                 /* Lowest Precedence */
%left   L_OR
%left   L_AND
%left   '|' B_NOR
%left   B_EQU '^'
%left   B_NAND '&'
%left   L_EQ C_EQ L_NEQ C_NEQ
%left   '>' '<' GTE LTE
%left   LSL LSR ASR ASL
%left   '+' '-'
%left   '*' '/' '%'
%left   POW
%right  '!' '~'    /* Highest Precedence. */
				
%start grammar_begin

%type <ival> signed_o net_type output_variable_type_o output_variable_type
             strength0 strength1 charge_strength
			 automatic_o
			 task_port_type_o task_port_type
			 gatetype_n_output
			 enable_gatetype
			 gatetype_n_input

%type <treenode> module_identifier 
             attr_spec attr_name constant_expression identifier 
			 escaped_identifier simple_identifier
             parameter_declaration local_parameter_declaration
			 range range_o dimension
			 port port_identifier port_reference range_expression
%type <list> attribute_instance_list list_of_attribute_instance_list attr_specs
			 module_parameter_port_list module_param_list
			 list_of_port_declarations  
			 list_of_ports ports 
			 port_expression
             non_port_module_item_list module_item_list module_params
			 list_of_param_assignments

%type <operator> unary_operator unary_module_path_operator binary_module_path_operator

%%

/* Start variables */
grammar_begin : 
  library_text {}
| source_text {}
| {}
;

/* A.1.1 Library Source Text */

library_text : 
  library_descriptions {}
| library_text library_descriptions{}
;

library_descriptions : 
  library_declaration{}
| include_statement{}
| config_declaration{}
;

library_declaration : 
  KW_LIBRARY library_identifier file_path_specs ';'{}
| KW_LIBRARY library_identifier file_path_specs KW_INCDIR file_path_specs ';'{}
;

file_path_specs : 
  file_path_spec{}
| file_path_specs ',' file_path_spec{}
;

file_path_spec : file_path{}
;

file_path : string {}
;

include_statement : KW_INCLUDE file_path_spec ';'{}
;
   
/* A.1.2  Verilog source text. */

source_text : 
  description {
  }
| source_text description{
}
;

description : 
  module_declaration {
  }
| udp_declaration {
}
| config_declaration {
}
;

module_declaration : 
  attribute_instance_list
  module_keyword
  module_identifier
  module_parameter_port_list
  list_of_ports
  ';'
  module_item_list
  KW_ENDMODULE
{
	IVerilogRoot ** ppRoot = getVerilogRoot();
	(*ppRoot)->add_module(ppRoot, verilogparseCreateModuleDeclaration(&$1,$3,&$4,$5,&$7));
}  

| attribute_instance_list
  module_keyword
  module_identifier
  module_parameter_port_list
  list_of_port_declarations
  ';'
  non_port_module_item_list
  KW_ENDMODULE
  {
	IVerilogRoot ** ppRoot = getVerilogRoot();
	(*ppRoot)->add_module(ppRoot, verilogparseCreateModuleDeclaration(&$1,$3,&$4,&$5,&$7));
}
;

module_keyword :
  KW_MODULE {
  }
| KW_MACROMODULE {
  }
  ;


/*
A.1.3 Module parameters and ports
*/

module_parameter_port_list  : {
}
| '#' '(' module_param_list ')'{
    $$ = $3;
}
;

module_param_list     : 
  parameter_declaration{
    dlistInit(&$$);
	dlistAppendItem(&$$, $1);
  }
| module_param_list ',' parameter_declaration{
	dlistAppendItem(&$1, $3);
	$$ = $1;
}
;

list_of_ports   : {
}
| '(' ports ')' {
    $$ = $2;
}
;

ports           : {
	dlistInit(&$$);
}
| ports ',' port{
    $$ = $1;
    dlistAppendItem(&$$,$3);
}
| port {
    dlistInit(&$$);
	dlistAppendItem(&$$,$1);
}
;

list_of_port_declarations   : 
  '(' ')'{
	dlistInit(&$$);
  }
| '(' port_declarations ')'{
    $$ = $2;
}
;

port_declarations : 
  { 
    dlistInit(&$$);
  }
| port_declarations ',' port_declaration {
    $$ = $1;
    dlistAppendItem(&$$,$3);
  }
| port_declaration {
    dlistInit(&$$);
	dlistAppendItem(&$$,$1);
  }
  ;

port            : 
  port_expression{
    //$$ = verilogparserCreatePort(NULL, &$1);
  }
| '.' port_identifier '(' port_expression ')'{
   // $$ = verilogparserCreatePort($2, &$4);
}
;

port_expression : 
	{
		dlistInit(&$$);
	}
| port_reference {
    dlistInit(&$$);
    dlistAppendItem(&$$, $1);
  }
| port_expression ',' port_reference{
    $$ = $1;
    dlistAppendItem(&$$,$3);
}
;

port_reference  : 
  port_identifier{
}
| port_identifier '[' ']' {
}
| port_identifier '[' constant_range_expression ']' {
}
;

port_declaration : 
    attribute_instance_list inout_declaration {
    }
|   attribute_instance_list input_declaration {
    }
|   attribute_instance_list output_declaration {
    }
;

/* 
A.1.4 Module Items 
*/

module_item :
    port_declaration {
    }
|   non_port_module_item {
    }
 ;

module_or_generate_item :
  attribute_instance_list module_or_generate_item_declaration {
  }
| attribute_instance_list local_parameter_declaration ';' {
  }
| attribute_instance_list parameter_override {
  }
| attribute_instance_list continuous_assign {
  }
| attribute_instance_list gate_instantiation {
  }
| attribute_instance_list udp_instantiation {
  }
| attribute_instance_list module_instantiation {
  }
| attribute_instance_list initial_construct {
  }
| attribute_instance_list always_construct {
  }
| attribute_instance_list loop_generate_construct {
  }
| attribute_instance_list conditional_generate_construct {
  }
  ;

module_or_generate_item_declaration :
    net_declaration {
    }
|   reg_declaration {
    }
|   integer_declaration {
    }
| real_declaration {
    }
| time_declaration {
    }
| realtime_declaration {
    }
| event_declaration {
    }
| genvar_declaration {
    }
| task_declaration {
    }
| function_declaration {
    }
 ;

non_port_module_item :
    module_or_generate_item {
    }
|   generate_region {
    }
| specify_block {
    }
|   attribute_instance_list parameter_declaration ';' {
    }
|   attribute_instance_list specparam_declaration {
    }
   ;

parameter_override :
    defparam list_of_defparam_assignments ';' {
    }
    ;

/*
A.1.5 Configuration source text
*/
config_declaration :
    config config_identifier ';' 
    design_statement endconfig {
    }
|   config config_identifier ';' 
    design_statement config_rule_statement endconfig {
    }
 ;

design_statement : design design_cell_list ';' {
}
;

design_cell_list :
  {
  }
|  design_cell_list design_cell {
  } 
|  design_cell {
 }
 ;
 
design_cell :
  cell_identifier {
  }
| library_identifier '.' cell_identifier {
  }
  ;

config_rule_statement :
    default_clause liblist_clause ';' {
    }
|   inst_clause liblist_clause ';' {
    }
|   inst_clause use_clause ';' {
    }
|   cell_clause liblist_clause ';' {
    }
|   cell_clause use_clause ';' {
    }
 ;

default_clause :
  default {
  }
  ;

inst_clause :
    instance inst_name {
    }
  ;

inst_name :
  topmodule_identifier {
  }
  inst_name '.' instance_identifier {
  }
  ;

cell_clause : 
    cell cell_identifier {
    }
|   cell library_identifier '.' cell_identifier {
    }
 ;

liblist_clause :
    liblist {
    }
|   liblist library_identifier {
    }
 ;

use_clause :
    use cell_identifier {
    }
|   use cell_identifier ':' config {
    }
|   use library_identifier '.' cell_identifier {
    }
|   use library_identifier '.' cell_identifier ':' config {
    }
  ;

/*
A.2 Declarations
A.2.1 Declaration types
A.2.1.1 Module parameter declarations
*/

signed_option: 
 {
 }
| KW_SIGNED {
}
;

range_option: 
 {
 }
| range {
}
;

local_parameter_declaration :
    localparam signed_option range_option list_of_param_assignments {
    }
|   localparam parameter_type list_of_param_assignments{
    }
 ;

parameter_declaration :
    parameter signed_option range_option list_of_param_assignments {
    }
|   parameter parameter_type list_of_param_assignments {
    }
;

specparam_declaration : 
    specparam range_option list_of_specparam_assignments ';' {
    }
   ;

parameter_type :
    integer {
    }
|   real {
    } 
|   realtime {
    }
|   time {
    }
 ;

/*
A.2.1.2 Port declarations
*/

net_type_option :
  {
  }
| net_type {
  }
;

inout_declaration :
    inout net_type_option signed_option range_option 
    list_of_port_identifiers {
    }
  ;

input_declaration :
    input net_type_option signed_option range_option 
    list_of_port_identifiers {
    }
  ;

output_declaration :
    output net_type_option signed_option range_option 
    list_of_port_identifiers {
    }
|   output reg signed_option range_option
    list_of_variable_port_identifiers {
    }
|   output output_variable_type 
    list_of_variable_port_identifiers {
    }
  ;

/*
A.2.1.3 Type declarations
*/
event_declaration :
    event list_of_event_identifiers ';' {
    }
 ;

integer_declaration : integer list_of_variable_identifiers ';' {
    }
  ;
delay3_option: 
    {
    }
|   delay3 {
    }
  ;

drive_strength_option :
    {
    }
|   drive_strength {
    }
  ;

vectored_option :
    {
    }
|   vectored {
    }
  ;

scalared_option :
    {
    }
|   scalared {
    }
 ;

charge_strength_option :
    {
    }
|   charge_strength {
    }
 ;

vectored_or_scalared_option : 
    {
    } 
|   vectored {
    }
|   scalared {
    }
 ;

net_declaration :
    net_type signed_option
    delay3_option list_of_net_identifiers ';' {
    }
|   net_type drive_strength_option signed_option
    delay3_option list_of_net_decl_assignments ';' {
    }
|   net_type vectored_option signed_option
    range delay3_option list_of_net_identifiers ';' {
    }
|   net_type scalared_option signed_option
    range delay3_option list_of_net_identifiers ';' {
    }
|   net_type drive_strength_option vectored_option 
    scalared_option signed_option
    range delay3_option list_of_net_decl_assignments ';' {
    }
|   trireg charge_strength_option signed_optin
    delay3_option list_of_net_identifiers ';'
|   trireg drive_strength_option signed_option
    delay3_option list_of_net_decl_assignments ';' {
    }
|   trireg charge_strength_option vectored_or_scalared_option signed_option
    range delay3_option list_of_net_identifiers ';' {
    }
|   trireg drive_strength_option vectored_or_scalared_option signed_option
    range delay3_option list_of_net_decl_assignments ';' {
    }
 ;

real_declaration :
    real list_of_real_identifiers ';' {
    }
  ;

realtime_declaration : 
    realtime list_of_real_identifiers ';' {
    }
  ;

reg_declaration :
    reg signed_option range_option
    list_of_variable_identifiers ';' {
    }
  ;

time_declaration : 
    time list_of_variable_identifiers ';' {
    }
  ;

/*
A.2.2 Declaration data types
A.2.2.1 Net and variable types
*/
net_type :
    supply0 {
    }
|   supply1 {
    }
|   tri {
    } 
|   triand  {
    }
|   trior  {
    }
|   tri0  {
    }
|   tri1 {
    }
|   uwire  {
    }
|   wire  {
    }
|   wand  {
    }
|   wor {
    }
  ;

output_variable_type : 
    integer  {
    }
|   time {
    }
 ;

dimension_list :
    {
    }
| dimension_list dimension {
    }
| dimension {
}
;

real_type :
    real_identifier dimension_list {
    }
|   real_identifier '=' constant_expression {
    }
 ;
variable_type :
    variable_identifier dimension_list {
    }
|   variable_identifier '=' constant_expression {
    }
 ;
 
/*
A.2.2.2 Strengths 
*/

drive_strength :
    '(' strength0 ',' strength1 ')' {
    }
|   '(' strength1 ',' strength0 ')' {
    }
|   '(' strength0 ',' highz1 ')' {
    }
|   '(' strength1 ',' highz0 ')' {
    }
|   '(' highz0 ',' strength1 ')' {
    }
|   '(' highz1 ',' strength0 ')' {
    }
  ;
strength0 : 
    supply0  {
    }
|   strong0  {
    }
|   pull0  {
    }
|   weak0 {
    }
 ;

strength1 : 
    supply1  {
    }
|   strong1  {
    }
|   pull1  {
    }
|   weak1 {
    }
  ;

charge_strength : 
    '(' small ')' {
    } 
|   '(' medium ')'  {
    }
|   '(' large ')' {
    }
 ;

/*
A.2.2.3 Delays
*/

delay3 :
    '#' delay_value {
    }
|   '#' '(' mintypmax_expression ')' {
    }
|   '#' '(' mintypmax_expression ',' mintypmax_expression ')' {
    }
|   '#' '(' mintypmax_expression ',' mintypmax_expression ',' mintypmax_expression ')' {
    }
 ;

delay2 :
    '#' delay_value
|   '#' '(' mintypmax_expression ')' {
    }
|   '#' '(' mintypmax_expression ',' mintypmax_expression ')' {
    }
 ;

delay_value :
    unsigned_number {
    }
|   real_number {
    }
|   identifier {
    }
 ;

/*
A.2.3 Declaration lists
*/
list_of_defparam_assignments :
    defparam_assignment {
    }
|   list_of_defparam_assignments  ',' defparam_assignment {
    }
 ;

list_of_dimensions:
    dimension {
    }
|   list_of_dimensions  ',' dimension {
    }
 ;

list_of_event_identifiers : 
    event_identifier list_of_dimensions {
    }
|   list_of_event_identifiers ',' event_identifier list_of_dimensions {
    }
  ;

list_of_net_decl_assignments :
    net_decl_assignment {
    }
|   list_of_net_decl_assignments  ',' net_decl_assignment {
    }
 ;
 
list_of_event_identifiers : 
    net_identifier list_of_dimensions {
    }
|   list_of_event_identifiers ',' net_identifier list_of_dimensions {
    }
  ;

list_of_param_assignments :
    param_assignment {
    }
|   list_of_param_assignments  ',' param_assignment {
    }
 ;
 

list_of_port_identifiers :
    port_identifier {
    }
|   list_of_port_identifiers  ',' port_identifier {
    }
 ;
 
list_of_real_identifiers :
    real_type {
    }
|   list_of_real_identifiers  ',' real_type {
    }
 ;
 

list_of_specparam_assignments :
    specparam_assignment {
    }
|   list_of_specparam_assignments  ',' specparam_assignment {
    }
 ;
 
list_of_variable_identifiers :
    variable_type {
    }
|   list_of_variable_identifiers  ',' variable_type {
    }
 ;
 
port_ident :
    port_identifier {
    }
|   port_identifier '=' constant_expression {
    }
 ;

list_of_variable_port_identifiers :
    port_ident {
    }
|   list_of_variable_port_identifiers  ',' port_ident {
    }
 ;

/*
A.2.4 Declaration assignments
*/

defparam_assignment : 
    hierarchical_parameter_identifier '=' constant_mintypmax_expression {
    }
  ;

net_decl_assignment : 
    net_identifier '=' expression {
    }
  ;

param_assignment : 
    parameter_identifier '=' constant_mintypmax_expression {
    }
  ;

specparam_assignment :
    specparam_identifier '=' constant_mintypmax_expression {
    }
|    pulse_control_specparam {
    }
 ;

pulse_control_specparam : 
  {
  }

/*
    PATHPULSE$ '=' '(' reject_limit_value')' {
    }
|   PATHPULSE$ '=' '(' reject_limit_value ',' error_limit_value ')' {
    }
|   PATHPULSE$specify_input_terminal_descriptor$specify_output_terminal_descriptor
    '=' '(' reject_limit_value ')' {
    }
|   PATHPULSE$specify_input_terminal_descriptor$specify_output_terminal_descriptor
    '=' '(' reject_limit_value ',' error_limit_value ')' {
    }
*/
 ;

error_limit_value : 
    limit_value {
    }
  ;

reject_limit_value : 
    limit_value {
    }
  ;

limit_value : 
    constant_mintypmax_expression {
    }
 ;

/*
A.2.5 Declaration ranges
*/
dimension : 
    '[' dimension_constant_expression ':' dimension_constant_expression ']' {
    }
 ;

range : '[' msb_constant_expression ':' lsb_constant_expression ']' {
    }
  ;

/*
A.2.6 Function declarations
*/
automatic_option :
    {
    }
|   automatic { 
    }
  ;

function_range_or_type_option:
    {
    }
|   function_range_or_type {
    }
  ;

function_item_declaration_list:
    function_item_declaration {
    }
|   function_item_declaration_list function_item_declaration {
    }
 ;

block_item_declaration_list:
    {
    }
|   block_item_declaration {
    }
|   block_item_declaration_list block_item_declaration {
    }
 ;

function_declaration :
    function automatic_option function_range_or_type_option function_identifier ';'
    function_item_declaration_list
    function_statement
    endfunction {
    }
|   function automatic_option function_range_or_type_list function_identifier '(' function_port_list ')' ';'
    block_item_declaration_list   
    function_statement
    endfunction {
    }
 ;

function_item_declaration :
    block_item_declaration
|   attribute_instnce_list tf_input_declaration ';' {
    }
 ;

function_port_list :
    attribute_instance_list tf_input_declaration {
    }
|   function_port_list attribute_instance_list tf_input_declaration {
    }
  ;

function_range_or_type :
    signed_option range_option {
    }
|   integer {
    }
|   real {
    }
|   realtime {
    }
| time {
    }
  ;

/*
A.2.7 Task declarations
*/
task_item_declaration_list :
    {
    }
|   task_item_declaration_list task_item_declaration {
    }
|   task_item_declaration {
    }
 ;

task_port_list_option:
    {
    }
|   task_port_list {
    }
  ;

task_declaration :
    task automatic_option task_identifier ';'
    task_item_declaration_list
    statement_or_null
    endtask {
    }
|   task automatic_option task_identifier '(' task_port_list_option ')' ';'
    block_item_declaration_list
    statement_or_null
    endtask {
    }
 ;

task_item_declaration :
    block_item_declaration {
    }
|   attribute_instance_list tf_input_declaration ';' {
    }
|   attribute_instance_list tf_output_declaration ';' {
    }
|   attribute_instance_list tf_inout_declaration ';' {
    }
  ;

task_port_list :
    task_port_item {
    }
|   task_port_list ',' task_port_item {
    }
 ;

task_port_item :
    attribute_instance_list tf_input_declaration {
    }
|   attribute_instance_list tf_output_declaration {
    }
|   attribute_instance_list tf_inout_declaration {
    }
  ;

tf_input_declaration :
    input reg_option signed_option range_option list_of_port_identifiers {
    }
|   input task_port_type list_of_port_identifiers {
    }
  ;

tf_output_declaration :
    output reg_option signed_option range_option list_of_port_identifiers {
    }
|   output task_port_type list_of_port_identifiers {
    }
  ;

tf_inout_declaration :
    inout reg_option signed_option rangeoption list_of_port_identifiers {
    }
|   inout task_port_type list_of_port_identifiers {
    }
  ;

task_port_type :
    integer  {
    }
|   real {
    }
|   realtime {
    }
|   time{
    }
  ;

/*
A.2.8 Block item declarations
*/
block_item_declaration ::=
{ attribute_instance } reg [ signed ] [ range ] list_of_block_variable_identifiers ;
| { attribute_instance } integer list_of_block_variable_identifiers ;
| { attribute_instance } time list_of_block_variable_identifiers ;
| { attribute_instance } real list_of_block_real_identifiers ;
| { attribute_instance } realtime list_of_block_real_identifiers ;
| { attribute_instance } event_declaration
| { attribute_instance } local_parameter_declaration ;
| { attribute_instance } parameter_declaration ;
list_of_block_variable_identifiers ::= block_variable_type { , block_variable_type }
list_of_block_real_identifiers ::= block_real_type { , block_real_type }
block_variable_type ::= variable_identifier { dimension }
block_real_type ::= real_identifier { dimension }
/*
A.3 Primitive instances
A.3.1 Primitive instantiation and instances
*/
gate_instantiation ::=
cmos_switchtype [delay3]
cmos_switch_instance { , cmos_switch_instance } ;
| enable_gatetype [drive_strength] [delay3]
enable_gate_instance { , enable_gate_instance } ;
| mos_switchtype [delay3]
mos_switch_instance { , mos_switch_instance } ;
| n_input_gatetype [drive_strength] [delay2]
n_input_gate_instance { , n_input_gate_instance } ;
| n_output_gatetype [drive_strength] [delay2]
n_output_gate_instance { , n_output_gate_instance } ;
| pass_en_switchtype [delay2]
pass_enable_switch_instance { , pass_enable_switch_instance } ;
| pass_switchtype
pass_switch_instance { , pass_switch_instance } ;
| pulldown [pulldown_strength]
pull_gate_instance { , pull_gate_instance } ;
| pullup [pullup_strength]
pull_gate_instance { , pull_gate_instance } ;
cmos_switch_instance ::= [ name_of_gate_instance ] ( output_terminal , input_terminal ,
ncontrol_terminal , pcontrol_terminal )
enable_gate_instance ::= [ name_of_gate_instance ] ( output_terminal , input_terminal , enable_terminal )
mos_switch_instance ::= [ name_of_gate_instance ] ( output_terminal , input_terminal , enable_terminal )
n_input_gate_instance ::= [ name_of_gate_instance ] ( output_terminal , input_terminal { , input_terminal } )
n_output_gate_instance ::= [ name_of_gate_instance ] ( output_terminal { , output_terminal } ,
input_terminal )
pass_switch_instance ::= [ name_of_gate_instance ] ( inout_terminal , inout_terminal )
pass_enable_switch_instance ::= [ name_of_gate_instance ] ( inout_terminal , inout_terminal ,
enable_terminal )
pull_gate_instance ::= [ name_of_gate_instance ] ( output_terminal )
name_of_gate_instance ::= gate_instance_identifier [ range ]
/*
A.3.2 Primitive strengths
*/
pulldown_strength ::=
( strength0 , strength1 )
| ( strength1 , strength0 )
| ( strength0 )
pullup_strength ::=
( strength0 , strength1 )
| ( strength1 , strength0 )
| ( strength1 )
/*
A.3.3 Primitive terminals
*/
enable_terminal ::= expression
inout_terminal ::= net_lvalue
input_terminal ::= expression
ncontrol_terminal ::= expression
output_terminal ::= net_lvalue
pcontrol_terminal ::= expression
/*
A.3.4 Primitive gate and switch types
*/
cmos_switchtype ::= cmos | rcmos
enable_gatetype ::= bufif0 | bufif1 | notif0 | notif1
mos_switchtype ::= nmos | pmos | rnmos | rpmos
n_input_gatetype ::= and | nand | or | nor | xor | xnor
n_output_gatetype ::= buf | not
pass_en_switchtype ::= tranif0 | tranif1 | rtranif1 | rtranif0
pass_switchtype ::= tran | rtran
/*
A.4 Module instantiation and generate construct
A.4.1 Module instantiation
*/
module_instantiation ::=
module_identifier [ parameter_value_assignment ]
module_instance { , module_instance } ;
parameter_value_assignment ::= # ( list_of_parameter_assignments )
list_of_parameter_assignments ::=
ordered_parameter_assignment { , ordered_parameter_assignment } |
named_parameter_assignment { , named_parameter_assignment }
ordered_parameter_assignment ::= expression
named_parameter_assignment ::= . parameter_identifier ( [ mintypmax_expression ] )
module_instance ::= name_of_module_instance ( [ list_of_port_connections ] )
name_of_module_instance ::= module_instance_identifier [ range ]
list_of_port_connections ::=
ordered_port_connection { , ordered_port_connection }
| named_port_connection { , named_port_connection }
ordered_port_connection ::= { attribute_instance } [ expression ]
named_port_connection ::= { attribute_instance } . port_identifier ( [ expression ] )
/*
A.4.2 Generate construct
*/
generate_region ::=
generate { module_or_generate_item } endgenerate
genvar_declaration ::=
genvar list_of_genvar_identifiers ;
list_of_genvar_identifiers ::=
genvar_identifier { , genvar_identifier }
loop_generate_construct ::=
for ( genvar_initialization ; genvar_expression ; genvar_iteration )
generate_block
genvar_initialization ::=
genvar_identifier = constant_expression
genvar_expression ::=
genvar_primary
| unary_operator { attribute_instance } genvar_primary
| genvar_expression binary_operator { attribute_instance } genvar_expression
| genvar_expression ? { attribute_instance } genvar_expression : genvar_expression
genvar_iteration ::=
genvar_identifier = genvar_expression
genvar_primary ::=
constant_primary
| genvar_identifier
conditional_generate_construct ::=
if_generate_construct
| case_generate_construct
if_generate_construct ::=
if ( constant_expression ) generate_block_or_null
[ else generate_block_or_null ]
case_generate_construct ::=
case ( constant_expression )
case_generate_item { case_generate_item } endcase
case_generate_item ::=
constant_expression { , constant_expression } : generate_block_or_null
| default [ : ] generate_block_or_null
generate_block ::=
module_or_generate_item
| begin [ : generate_block_identifier ] { module_or_generate_item } end
generate_block_or_null ::=
generate_block
| ;
/*
A.5 UDP declaration and instantiation
A.5.1 UDP declaration
*/
udp_declaration ::=
{ attribute_instance } primitive udp_identifier ( udp_port_list ) ;
udp_port_declaration { udp_port_declaration }
udp_body
endprimitive
| { attribute_instance } primitive udp_identifier ( udp_declaration_port_list ) ;
udp_body
endprimitive
/*
A.5.2 UDP ports
*/
udp_port_list ::= output_port_identifier , input_port_identifier { , input_port_identifier }
udp_declaration_port_list ::=
udp_output_declaration , udp_input_declaration { , udp_input_declaration }
udp_port_declaration ::=
udp_output_declaration ;
| udp_input_declaration ;
| udp_reg_declaration ;
udp_output_declaration ::=
{ attribute_instance } output port_identifier
| { attribute_instance } output reg port_identifier [ = constant_expression ]
udp_input_declaration ::= { attribute_instance } input list_of_port_identifiers
udp_reg_declaration ::= { attribute_instance } reg variable_identifier
/*
A.5.3 UDP body
*/
udp_body ::= combinational_body | sequential_body
combinational_body ::= table combinational_entry { combinational_entry } endtable
combinational_entry ::= level_input_list : output_symbol ;
sequential_body ::= [ udp_initial_statement ] table sequential_entry { sequential_entry } endtable
udp_initial_statement ::= initial output_port_identifier = init_val ;
init_val ::= 1'b0 | 1'b1 | 1'bx | 1'bX | 1'B0 | 1'B1 | 1'Bx | 1'BX | 1 | 0
sequential_entry ::= seq_input_list : current_state : next_state ;
seq_input_list ::= level_input_list | edge_input_list
level_input_list ::= level_symbol { level_symbol }
edge_input_list ::= { level_symbol } edge_indicator { level_symbol }
edge_indicator ::= ( level_symbol level_symbol ) | edge_symbol
current_state ::= level_symbol
next_state ::= output_symbol | -
output_symbol ::= 0 | 1 | x | X
level_symbol ::= 0 | 1 | x | X | ? | b | B
edge_symbol ::= r | R | f | F | p | P | n | N | *
/*
A.5.4 UDP instantiation
*/
udp_instantiation ::= udp_identifier [ drive_strength ] [ delay2 ]
udp_instance { , udp_instance } ;
udp_instance ::= [ name_of_udp_instance ] ( output_terminal , input_terminal
{ , input_terminal } )
name_of_udp_instance ::= udp_instance_identifier [ range ]
/*
A.6 Behavioral statements
A.6.1 Continuous assignment statements
*/
continuous_assign ::= assign [ drive_strength ] [ delay3 ] list_of_net_assignments ;
list_of_net_assignments ::= net_assignment { , net_assignment }
net_assignment ::= net_lvalue = expression
/*
A.6.2 Procedural blocks and assignments
*/
initial_construct ::= initial statement
always_construct ::= always statement
blocking_assignment ::= variable_lvalue = [ delay_or_event_control ] expression
nonblocking_assignment ::= variable_lvalue <= [ delay_or_event_control ] expression
procedural_continuous_assignments ::=
assign variable_assignment
| deassign variable_lvalue
| force variable_assignment
| force net_assignment
| release variable_lvalue
| release net_lvalue
variable_assignment ::= variable_lvalue = expression
/*
A.6.3 Parallel and sequential blocks
*/
par_block ::= fork [ : block_identifier
{ block_item_declaration } ] { statement } join
seq_block ::= begin [ : block_identifier
{ block_item_declaration } ] { statement } end
/*
A.6.4 Statements
*/
statement ::=
{ attribute_instance } blocking_assignment ;
| { attribute_instance } case_statement
| { attribute_instance } conditional_statement
| { attribute_instance } disable_statement
| { attribute_instance } event_trigger
| { attribute_instance } loop_statement
| { attribute_instance } nonblocking_assignment ;
| { attribute_instance } par_block
| { attribute_instance } procedural_continuous_assignments ;
| { attribute_instance } procedural_timing_control_statement
| { attribute_instance } seq_block
| { attribute_instance } system_task_enable
| { attribute_instance } task_enable
| { attribute_instance } wait_statement
statement_or_null ::=
statement
| { attribute_instance } ;
function_statement1 ::= statement
/*
A.6.5 Timing control statements
*/
delay_control ::=
# delay_value
| # ( mintypmax_expression )
delay_or_event_control ::=
delay_control
| event_control
| repeat ( expression ) event_control
disable_statement ::=
disable hierarchical_task_identifier ;
| disable hierarchical_block_identifier ;
event_control ::=
@ hierarchical_event_identifier
| @ ( event_expression )
| @*
| @ (*)
event_trigger ::=
-> hierarchical_event_identifier { [ expression ] } ;
event_expression ::=
expression
| posedge expression
| negedge expression
| event_expression or event_expression
| event_expression , event_expression
procedural_timing_control ::=
delay_control
| event_control
procedural_timing_control_statement ::=
procedural_timing_control statement_or_null
wait_statement ::=
wait ( expression ) statement_or_null
/*
A.6.6 Conditional statements
*/
conditional_statement ::=
if ( expression )
statement_or_null [ else statement_or_null ]
| if_else_if_statement
if_else_if_statement ::=
if ( expression ) statement_or_null
{ else if ( expression ) statement_or_null }
[ else statement_or_null ]
/*
A.6.7 Case statements
*/
case_statement ::=
case ( expression )
case_item { case_item } endcase
| casez ( expression )
case_item { case_item } endcase
| casex ( expression )
case_item { case_item } endcase
case_item ::=
expression { , expression } : statement_or_null
| default [ : ] statement_or_null
/*
A.6.8 Looping statements
*/
loop_statement ::=
forever statement
| repeat ( expression ) statement
| while ( expression ) statement
| for ( variable_assignment ; expression ; variable_assignment )
statement
/*
A.6.9 Task enable statements
*/
system_task_enable ::= system_task_identifier [ ( [ expression ] { , [ expression ] } ) ] ;
task_enable ::= hierarchical_task_identifier [ ( expression { , expression } ) ] ;
/*
A.7 Specify section
A.7.1 Specify block declaration
*/
specify_block ::= specify { specify_item } endspecify
specify_item ::=
specparam_declaration
| pulsestyle_declaration
| showcancelled_declaration
| path_declaration
| system_timing_check
pulsestyle_declaration ::=
pulsestyle_onevent list_of_path_outputs ;
| pulsestyle_ondetect list_of_path_outputs ;
showcancelled_declaration ::=
showcancelled list_of_path_outputs ;
| noshowcancelled list_of_path_outputs ;
/*
A.7.2 Specify path declarations
*/
path_declaration ::=
simple_path_declaration ;
| edge_sensitive_path_declaration ;
| state_dependent_path_declaration ;
simple_path_declaration ::=
parallel_path_description = path_delay_value
| full_path_description = path_delay_value
parallel_path_description ::=
( specify_input_terminal_descriptor [ polarity_operator ] => specify_output_terminal_descriptor )
full_path_description ::=
( list_of_path_inputs [ polarity_operator ] *> list_of_path_outputs )
list_of_path_inputs ::=
specify_input_terminal_descriptor { , specify_input_terminal_descriptor }
list_of_path_outputs ::=
specify_output_terminal_descriptor { , specify_output_terminal_descriptor }
/*
A.7.3 Specify block terminals
*/
specify_input_terminal_descriptor ::=
input_identifier [ [ constant_range_expression ] ]
specify_output_terminal_descriptor ::=
output_identifier [ [ constant_range_expression ] ]
input_identifier ::= input_port_identifier | inout_port_identifier
output_identifier ::= output_port_identifier | inout_port_identifier
/*
A.7.4 Specify path delays
*/
path_delay_value ::=
list_of_path_delay_expressions
| ( list_of_path_delay_expressions )
list_of_path_delay_expressions ::=
t_path_delay_expression
| trise_path_delay_expression , tfall_path_delay_expression
| trise_path_delay_expression , tfall_path_delay_expression , tz_path_delay_expression
| t01_path_delay_expression , t10_path_delay_expression , t0z_path_delay_expression ,
tz1_path_delay_expression , t1z_path_delay_expression , tz0_path_delay_expression
| t01_path_delay_expression , t10_path_delay_expression , t0z_path_delay_expression ,
tz1_path_delay_expression , t1z_path_delay_expression , tz0_path_delay_expression ,
t0x_path_delay_expression , tx1_path_delay_expression , t1x_path_delay_expression ,
tx0_path_delay_expression , txz_path_delay_expression , tzx_path_delay_expression
t_path_delay_expression ::= path_delay_expression
trise_path_delay_expression ::= path_delay_expression
tfall_path_delay_expression ::= path_delay_expression
tz_path_delay_expression ::= path_delay_expression
t01_path_delay_expression ::= path_delay_expression
t10_path_delay_expression ::= path_delay_expression
t0z_path_delay_expression ::= path_delay_expression
tz1_path_delay_expression ::= path_delay_expression
t1z_path_delay_expression ::= path_delay_expression
tz0_path_delay_expression ::= path_delay_expression
t0x_path_delay_expression ::= path_delay_expression
tx1_path_delay_expression ::= path_delay_expression
t1x_path_delay_expression ::= path_delay_expression
tx0_path_delay_expression ::= path_delay_expression
txz_path_delay_expression ::= path_delay_expression
tzx_path_delay_expression ::= path_delay_expression
path_delay_expression ::= constant_mintypmax_expression
edge_sensitive_path_declaration ::=
parallel_edge_sensitive_path_description = path_delay_value
| full_edge_sensitive_path_description = path_delay_value
parallel_edge_sensitive_path_description ::=
( [ edge_identifier ] specify_input_terminal_descriptor =>
( specify_output_terminal_descriptor [ polarity_operator ] : data_source_expression ) )
full_edge_sensitive_path_description ::=
( [ edge_identifier ] list_of_path_inputs *>
( list_of_path_outputs [ polarity_operator ] : data_source_expression ) )
data_source_expression ::= expression
edge_identifier ::= posedge | negedge
state_dependent_path_declaration ::=
if ( module_path_expression ) simple_path_declaration
| if ( module_path_expression ) edge_sensitive_path_declaration
| ifnone simple_path_declaration
polarity_operator ::= + | -
/*
A.7.5 System timing checks
A.7.5.1 System timing check commands
*/
system_timing_check ::=
$setup_timing_check
| $hold_timing_check
| $setuphold_timing_check
| $recovery_timing_check
| $removal_timing_check
| $recrem_timing_check
| $skew_timing_check
| $timeskew_timing_check
| $fullskew_timing_check
| $period_timing_check
| $width_timing_check
| $nochange_timing_check
$setup_timing_check ::=
$setup ( data_event , reference_event , timing_check_limit [ , [ notifier ] ] ) ;
$hold_timing_check ::=
$hold ( reference_event , data_event , timing_check_limit [ , [ notifier ] ] ) ;
$setuphold_timing_check ::=
$setuphold ( reference_event , data_event , timing_check_limit , timing_check_limit
[ , [ notifier ] [ , [ stamptime_condition ] [ , [ checktime_condition ]
[ , [ delayed_reference ] [ , [ delayed_data ] ] ] ] ] ] ) ;
$recovery_timing_check ::=
$recovery ( reference_event , data_event , timing_check_limit [ , [ notifier ] ] ) ;
$removal_timing_check ::=
$removal ( reference_event , data_event , timing_check_limit [ , [ notifier ] ] ) ;
$recrem_timing_check ::=
$recrem ( reference_event , data_event , timing_check_limit , timing_check_limit
[ , [ notifier ] [ , [ stamptime_condition ] [ , [ checktime_condition ]
[ , [ delayed_reference ] [ , [ delayed_data ] ] ] ] ] ] ) ;
$skew_timing_check ::=
$skew ( reference_event , data_event , timing_check_limit [ , [ notifier ] ] ) ;
$timeskew_timing_check ::=
$timeskew ( reference_event , data_event , timing_check_limit
[ , [ notifier ] [ , [ event_based_flag ] [ , [ remain_active_flag ] ] ] ] ) ;
$fullskew_timing_check ::=
$fullskew ( reference_event , data_event , timing_check_limit , timing_check_limit
[ , [ notifier ] [ , [ event_based_flag ] [ , [ remain_active_flag ] ] ] ] ) ;
$period_timing_check ::=
$period ( controlled_reference_event , timing_check_limit [ , [ notifier ] ] ) ;
$width_timing_check ::=
$width ( controlled_reference_event , timing_check_limit
[ , threshold [ , notifier ] ] ) ;
$nochange_timing_check ::=
$nochange ( reference_event , data_event , start_edge_offset ,
end_edge_offset [ , [ notifier ] ] ) ;
/*
A.7.5.2 System timing check command arguments
*/
checktime_condition ::= mintypmax_expression
controlled_reference_event ::= controlled_timing_check_event
data_event ::= timing_check_event
delayed_data ::=
terminal_identifier
| terminal_identifier [ constant_mintypmax_expression ]
delayed_reference ::=
terminal_identifier
| terminal_identifier [ constant_mintypmax_expression ]
end_edge_offset ::= mintypmax_expression
event_based_flag ::= constant_expression
notifier ::= variable_identifier
reference_event ::= timing_check_event
remain_active_flag ::= constant_expression
stamptime_condition ::= mintypmax_expression
start_edge_offset ::= mintypmax_expression
threshold ::= constant_expression
timing_check_limit ::= expression
/*
A.7.5.3 System timing check event definitions
*/
timing_check_event ::=
[timing_check_event_control] specify_terminal_descriptor [ &&& timing_check_condition ]
controlled_timing_check_event ::=
timing_check_event_control specify_terminal_descriptor [ &&& timing_check_condition ]
timing_check_event_control ::=
posedge
| negedge
| edge_control_specifier
specify_terminal_descriptor ::=
specify_input_terminal_descriptor
| specify_output_terminal_descriptor
edge_control_specifier ::= edge [ edge_descriptor { , edge_descriptor } ]
edge_descriptor2 ::=
01
| 10
| z_or_x zero_or_one
| zero_or_one z_or_x
zero_or_one ::= 0 | 1
z_or_x ::= x | X | z | Z
timing_check_condition ::=
scalar_timing_check_condition
| ( scalar_timing_check_condition )
scalar_timing_check_condition ::=
expression
| ~ expression
| expression == scalar_constant
| expression === scalar_constant
| expression != scalar_constant
| expression !== scalar_constant
scalar_constant ::=
1'b0 | 1'b1 | 1'B0 | 1'B1 | 'b0 | 'b1 | 'B0 | 'B1 | 1 | 0
/*
A.8 Expressions
A.8.1 Concatenations
*/
concatenation ::= { expression { , expression } }
constant_concatenation ::= { constant_expression { , constant_expression } }
constant_multiple_concatenation ::= { constant_expression constant_concatenation }
module_path_concatenation ::= { module_path_expression { , module_path_expression } }
module_path_multiple_concatenation ::= { constant_expression module_path_concatenation }
multiple_concatenation ::= { constant_expression concatenation }
/*
A.8.2 Function calls
*/
constant_function_call ::= function_identifier { attribute_instance }
( constant_expression { , constant_expression } )
constant_system_function_call ::= system_function_identifier
( constant_expression { , constant_expression } )
function_call ::= hierarchical_function_identifier{ attribute_instance }
( expression { , expression } )
system_function_call ::= system_function_identifier
[ ( expression { , expression } ) ]
/*
A.8.3 Expressions
*/
base_expression ::= expression
conditional_expression ::= expression1 ? { attribute_instance } expression2 : expression3
constant_base_expression ::= constant_expression
constant_expression ::=
constant_primary
| unary_operator { attribute_instance } constant_primary
| constant_expression binary_operator { attribute_instance } constant_expression
| constant_expression ? { attribute_instance } constant_expression : constant_expression
constant_mintypmax_expression ::=
constant_expression
| constant_expression : constant_expression : constant_expression
constant_range_expression ::=
constant_expression
| msb_constant_expression : lsb_constant_expression
| constant_base_expression +: width_constant_expression
| constant_base_expression -: width_constant_expression
dimension_constant_expression ::= constant_expression
expression ::=
primary
| unary_operator { attribute_instance } primary
| expression binary_operator { attribute_instance } expression
| conditional_expression
expression1 ::= expression
expression2 ::= expression
expression3 ::= expression
lsb_constant_expression ::= constant_expression
mintypmax_expression ::=
expression
| expression : expression : expression
module_path_conditional_expression ::= module_path_expression ? { attribute_instance }
module_path_expression : module_path_expression
module_path_expression ::=
module_path_primary
| unary_module_path_operator { attribute_instance } module_path_primary
| module_path_expression binary_module_path_operator { attribute_instance }
module_path_expression
| module_path_conditional_expression
module_path_mintypmax_expression ::=
module_path_expression
| module_path_expression : module_path_expression : module_path_expression
msb_constant_expression ::= constant_expression
range_expression ::=
expression
| msb_constant_expression : lsb_constant_expression
| base_expression +: width_constant_expression
| base_expression -: width_constant_expression
width_constant_expression ::= constant_expression
/*
A.8.4 Primaries
*/
constant_primary ::=
number
| parameter_identifier [ [ constant_range_expression ] ]
| specparam_identifier [ [ constant_range_expression ] ]
| constant_concatenation
| constant_multiple_concatenation
| constant_function_call
| constant_system_function_call
| ( constant_mintypmax_expression )
| string
module_path_primary ::=
number
| identifier
| module_path_concatenation
| module_path_multiple_concatenation
| function_call
| system_function_call
| ( module_path_mintypmax_expression )
primary ::=
number
| hierarchical_identifier [ { [ expression ] } [ range_expression ] ]
| concatenation
| multiple_concatenation
| function_call
| system_function_call
| ( mintypmax_expression )
| string
/*
A.8.5 Expression left-side values
*/
net_lvalue ::=
hierarchical_net_identifier [ { [ constant_expression ] } [ constant_range_expression ] ]
| { net_lvalue { , net_lvalue } }
variable_lvalue ::=
hierarchical_variable_identifier [ { [ expression ] } [ range_expression ] ]
| { variable_lvalue { , variable_lvalue } }
/*
A.8.6 Operators
*/
unary_operator :
    '+' | '-' | '!' | '~' | '&' | ~& | '|' | ~| | '^' | ~^ | ^~
    ;

binary_operator :
    '+' | '-' | '*' | '/' | '%' | == | != | === | !== | && | || | **
    | '<' | <= | '>' | >= | '&' | '|' | '^' | ^~ | ~^ | >> | << | >>> | <<<
  ;

unary_module_path_operator :
    ! | ~ | & | ~& | | | ~| | ^ | ~^ | ^~
  ;

binary_module_path_operator :
    == | != | && | || | & | | | ^ | ^~ | ~^
 ;
/*
A.8.7 Numbers
*/
number ::=
decimal_number
| octal_number
| binary_number
| hex_number
| real_number
real_number ::=
unsigned_number . unsigned_number
| unsigned_number [ . unsigned_number ] exp [ sign ] unsigned_number
exp ::= e | E
decimal_number ::=
unsigned_number
| [ size ] decimal_base unsigned_number
| [ size ] decimal_base x_digit { _ }
| [ size ] decimal_base z_digit { _ }
binary_number ::= [ size ] binary_base binary_value
octal_number ::= [ size ] octal_base octal_value
hex_number ::= [ size ] hex_base hex_value
sign ::= + | -
size ::= non_zero_unsigned_number
non_zero_unsigned_number2 ::= non_zero_decimal_digit { _ | decimal_digit}
unsigned_number2 ::= decimal_digit { _ | decimal_digit }
binary_value2 ::= binary_digit { _ | binary_digit }
octal_value2 ::= octal_digit { _ | octal_digit }
hex_value2 ::= hex_digit { _ | hex_digit }
decimal_base2 ::= '[s|S]d | '[s|S]D
binary_base2 ::= '[s|S]b | '[s|S]B
octal_base2 ::= '[s|S]o | '[s|S]O
hex_base2 ::= '[s|S]h | '[s|S]H
non_zero_decimal_digit ::= 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
decimal_digit ::= 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
binary_digit ::= x_digit | z_digit | 0 | 1
octal_digit ::= x_digit | z_digit | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7
hex_digit ::=
x_digit | z_digit | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9
| a | b | c | d | e | f | A | B | C | D | E | F
x_digit ::= x | X
z_digit ::= z | Z | ?

/*
A.8.8 Strings
*/
/* string ::= " { Any_ASCII_Characters_except_new_line } " */
/*
A.9 General
A.9.1 Attributes
*/
attribute_instance_list :
    {
    }
|   attribute_instance_list attribute_instance {
    }
|   attribute_instance {
    }
 ;

attribute_instance : 
    ATTRIBUTE_START attribute_instance_spec_list ATTRIBUTE_END {
    }
 ;
attribute_instance_spec_list : 
    attribute_instance_spec_list ',' attr_spec {
    }
|   attr_spec {
    }
 ;

attr_spec :
    attr_name {
    }
|   attr_name  '=' constant_expression {
    }
  ;

attr_name : 
    identifier {
    }
 ;
/*
A.9.2 Comments
*/

/*
comment ::=
one_line_comment
| block_comment
one_line_comment ::= // comment_text \n
block_comment ::= / * comment_text * /
comment_text ::= { Any_ASCII_character }
*/

/*
A.9.3 Identifiers
*/

block_identifier : 
    identifier {
    }
 ;

cell_identifier : 
    identifier {
    }
  ;

config_identifier :
    identifier {
    }
  :

event_identifier : 
    identifier  {
    }
 ;

function_identifier :  
    identifier {
    }
 ;

gate_instance_identifier : 
    identifier  {
    }
 ;

generate_block_identifier : 
    identifier {
    }
 ;

genvar_identifier : 
    identifier {
    }
 ;

hierarchical_block_identifier : 
    hierarchical_identifier {
    }
  ;

hierarchical_event_identifier : 
    hierarchical_identifier {
    }
  ;

hierarchical_function_identifier : 
    hierarchical_identifier {
    }
  ;

hierachical_header:
    {
    }
|   identifier '.' {
    }
|   identifier '[' constant_expression ']' '.' {
    }
|   hierachical_header identifier '.' {
    }
|   hierachical_header identifier '[' constant_expression ']' '.' {
    }
 ;
    
hierarchical_identifier : 
  hierachical_header identifier {
  }
 ;

hierarchical_net_identifier : 
    hierarchical_identifier {
    }
 ;

hierarchical_parameter_identifier : 
    hierarchical_identifier {
    }
  ;

hierarchical_variable_identifier : 
    hierarchical_identifier {
    }
  ;

hierarchical_task_identifier : 
    hierarchical_identifier {
    }
  ;

identifier :
    simple_identifier {
    }
|   escaped_identifier {
    }
  ;

inout_port_identifier : 
    identifier {
    }
  ;

input_port_identifier : 
    identifier {
    }
  ;

instance_identifier : 
    identifier {
    }
  ;

library_identifier : 
    identifier {
    }
  ;

module_identifier : 
    identifier {
    }
  ;

module_instance_identifier : 
    identifier {
    }
 ;
net_identifier : identifier {
    }
  ;

output_port_identifier : 
    identifier {
    }
  ;

parameter_identifier : 
    identifier {
    }
  ;

port_identifier : 
    identifier {
    }
  ;

real_identifier : 
    identifier {
    }
  ;

specparam_identifier : 
    identifier {
    }
  ;

task_identifier : 
    identifier {
    }
  ;

terminal_identifier : 
    identifier {
    }
  ;

text_macro_identifier : 
    identifier {
    }
  ;

topmodule_identifier : 
    identifier {
    }
  ;

udp_identifier : 
    identifier {
    }
  ;

udp_instance_identifier : 
    identifier {
    }
  ;
variable_identifier : 
    identifier {
    }
  ;

/*
A.9.4 White space
*/
white_space : 
    space  {
    }
|   tab {
    }
|   newline {
    }
|   eof {
    }
  ;
%%
