%{
    #include "verilog_parser.h" 
	#include "object.h"
	#include "preprocess.h"   
    #include "verilog_preprocess.h"
    #include "verilog_keyword.h"

    static IPreprocess ** preprocess = NULL;
	static char * __macroname = NULL;
	 
	int SetPreProcess(HOBJECT object)
	{
		return objectQueryInterface(object, IID_PREPROCESS, (const void **)&preprocess);
	}

	extern int verilog_find_reserved_word(const char * ident);
	#define YY_NO_UNISTD_H

	#define YY_INPUT(buf, result, max_size) \
	{ \
		 result = objectCall2(preprocess, GetText, buf, max_size); \
	}

    #define EMIT_TOKEN(x) if (objectCall0(preprocess, SymbolEmitEnabled)) { return x; }
%}

%option yylineno
%option nodefault 
%option noyywrap 
 
/* Tokens related to numbers */

EXP                 "e"|"E"
UNDERSCORE          "_"
SIGN                "+"|"-"
X                   "x"|"X"
Z                   "z"|"Z"|"?"
DOT					"."

DIGIT_DECIMAL       [0-9]
DIGIT_DECMIAL_NZ    [1-9]
DIGIT_BINARY        [0-1]|{X}|{Z}
DIGIT_OCTAL         [0-7]|{X}|{Z}
DIGIT_HEX           [0-9a-fA-F]|{X}|{Z}
DIGIT_DEC			[0-9]|{X}|{Z}

BASE_DECIMAL        '[sS]?[dD]
BASE_BINARY         '[sS]?[bB]
BASE_OCTAL          '[sS]?[oO]
BASE_HEX            '[sS]?[hH]

NUM_REAL_EXP        {NUM_UNSIGNED}({DOT}{NUM_UNSIGNED})?{EXP}({SIGN})?{NUM_UNSIGNED}

BIN_VALUE           {DIGIT_BINARY}({UNDERSCORE}|{DIGIT_BINARY})*
OCT_VALUE           {DIGIT_OCTAL}({UNDERSCORE}|{DIGIT_OCTAL})*
HEX_VALUE           {DIGIT_HEX}({UNDERSCORE}|{DIGIT_HEX})*
DEC_VALUE           {DIGIT_DEC}({UNDERSCORE}|{DIGIT_DEC})*

%x in_dec_val
%x in_hex_val
%x in_oct_val
%x in_bin_val
%s in_number

NUM_REAL            {NUM_UNSIGNED}{DOT}{NUM_UNSIGNED}|{NUM_REAL_EXP}
NUM_UNSIGNED        {DIGIT_DECIMAL}({UNDERSCORE}|{DIGIT_DECIMAL})*

/* Identifiers */

SYSTEM_ID           \$[a-zA-Z0-9_\$]+
SIMPLE_ID           [a-zA-Z_][a-zA-Z0-9_$]*
ESCAPED_ID          \\{SIMPLE_ID}

/* Strings */

STRING              \".*\"

/* Operators */

ASL                 "<<<"
ASR                 ">>>"
LSL                 "<<"
LSR                 ">>"
POW                 "**"
GTE                 ">="
LTE                 "<="
L_AND               "&&"
L_OR                "||"
C_EQ                "==="
L_EQ                "=="
C_NEQ               "!=="
L_NEQ               "!="
B_EQU               "^~"|"~^"
B_NAND              "~&"
B_NOR               "~|"

%%
"@"		{EMIT_TOKEN('@');}
","     {EMIT_TOKEN(',');}
"#"     {EMIT_TOKEN('#');}
"."     {EMIT_TOKEN('.');}
"="     {EMIT_TOKEN('=');}
":"     {EMIT_TOKEN(':');}
";"     {EMIT_TOKEN(';');}
"("     {EMIT_TOKEN('(');}
")"     {EMIT_TOKEN(')');}
"["     {EMIT_TOKEN('[');}
"]"     {EMIT_TOKEN(']');}
"{"     {EMIT_TOKEN('{');}
"}"     {EMIT_TOKEN('}');}
"+"     {EMIT_TOKEN('+');}
"-"     {EMIT_TOKEN('-');}
"*"     {EMIT_TOKEN('*');}
"/"     {EMIT_TOKEN('/');}
"%"     {EMIT_TOKEN('%');}
">"     {EMIT_TOKEN('>');}
"<"     {EMIT_TOKEN('<');}
"!"     {EMIT_TOKEN('!');}
"&"     {EMIT_TOKEN('&');}
"|"     {EMIT_TOKEN('|');}
"^"     {EMIT_TOKEN('^');}
"~"     {EMIT_TOKEN('~');}
"?"     {EMIT_TOKEN('?');}
"(*"    {EMIT_TOKEN(ATTRIBUTE_START);}
"*)"    {EMIT_TOKEN(ATTRIBUTE_END);}
"<<<"   {EMIT_TOKEN(ASL);}
">>>"   {EMIT_TOKEN(ASR);}
"<<"    {EMIT_TOKEN(LSL);}
">>"    {EMIT_TOKEN(LSR);}
">="    {EMIT_TOKEN(GTE);}
"<="    {EMIT_TOKEN(LTE);}
"&&"    {EMIT_TOKEN(L_AND);}
"||"    {EMIT_TOKEN(L_OR);}
"=="    {EMIT_TOKEN(L_EQ);}
"==="   {EMIT_TOKEN(C_EQ);}
"!="    {EMIT_TOKEN(L_NEQ);}
"!=="   {EMIT_TOKEN(C_NEQ);}
"^~"    {EMIT_TOKEN(B_EQU);}
"~^"    {EMIT_TOKEN(B_EQU);}
"~&"    {EMIT_TOKEN(B_NAND);}
"~|"    {EMIT_TOKEN(B_NOR);}
"+:"    {EMIT_TOKEN(IDX_PRT_SEL);}
"-:"    {EMIT_TOKEN(IDX_PRT_SEL);}

{BASE_DECIMAL}         {BEGIN(in_dec_val); EMIT_TOKEN(DEC_BASE);}
{BASE_HEX}             {BEGIN(in_hex_val); EMIT_TOKEN(HEX_BASE);}
{BASE_OCTAL}           {BEGIN(in_oct_val); EMIT_TOKEN(OCT_BASE);}
{BASE_BINARY}          {BEGIN(in_bin_val); EMIT_TOKEN(BIN_BASE);}

<in_bin_val>{BIN_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(BIN_VALUE);}
<in_oct_val>{OCT_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(OCT_VALUE);}
<in_hex_val>{HEX_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(HEX_VALUE);}
<in_dec_val>{DEC_VALUE} {BEGIN(INITIAL); yylval.string = _strdup(yytext); EMIT_TOKEN(DEC_VALUE);}


{NUM_REAL}             {yylval.string=_strdup(yytext);EMIT_TOKEN(NUM_REAL);}
{NUM_UNSIGNED}         {
    yylval.string=_strdup(yytext);
    EMIT_TOKEN(UNSIGNED_NUMBER);
	}

{SYSTEM_ID}            {
    /*yylval.identifier = ast_new_identifier(yytext,yylineno);*/
	yylval.string=_strdup(yytext); 
    EMIT_TOKEN(SYSTEM_ID);
}
{ESCAPED_ID}           {
    /*yylval.identifier = ast_new_identifier(yytext,yylineno);*/
	yylval.string=_strdup(yytext);
    EMIT_TOKEN(ESCAPED_ID);
}
{SIMPLE_ID}            {
    /*yylval.identifier = ast_new_identifier(yytext,yylineno);*/
	yylval.string=_strdup(yytext);
	if (objectCall0(preprocess, SymbolEmitEnabled)) {
		int id = verilog_find_reserved_word(yytext);
		if (id == -1) {
			EMIT_TOKEN(SIMPLE_ID);
		} else {
			EMIT_TOKEN(id);
		}
	}
}

{STRING}               {yylval.string= _strdup(yytext);EMIT_TOKEN(STRING);}

<*>"\n"              {/*EMIT_TOKEN(NEWLINE); IGNORE */   }
<*>" "                {/*EMIT_TOKEN(SPACE);   IGNORE */   }
<*>"\t"                  {/*EMIT_TOKEN(TAB);     IGNORE */   }

<<EOF>> {

    yypop_buffer_state();

    if ( !YY_CURRENT_BUFFER )
    {
        yyterminate();
    }
    else
    {
        YY_BUFFER_STATE cur = YY_CURRENT_BUFFER;
        yylineno = cur -> yy_bs_lineno;
    }
}

.                      {
    EMIT_TOKEN(ANY);
}

%%
